require 'spec_helper'

describe Formic::Input do
  before(:each) do
    @model = Object.new
    stub(@page = Object.new).url_for(@model) {'/models/1'}
    stub(@page).render

    @options = {}
    @block = Proc.new {}
  end

  context "#initialize" do
    before(:each) do
      @input = Formic::Input.new @page, 'input_name', @options
    end
    it "should have formic/default/input as default template" do
      @input.should have_template 'formic/default/input'
    end

    it "should have correct class name" do
      @input.should have_class 'input'
    end

    it "should have correct input name" do
      @input.should have_name 'input_name'
    end
  end

  context "interact with form" do
    context "form with model" do
      before(:each) do
        @form = Formic::Form.new @page
        @form.create @model
        mock(@model).send(:some_name) {'model value'}
        @input = Formic::Input.new @page, 'some_name', @options
      end

      it "should return the current form if input is inside a form" do
        @input.form.should == @form
      end

      it "should have name which is model[field]" do
        @input.should have_name 'object[some_name]'
      end

      it "should have value for field" do
        @input.field.should == 'some_name'
      end

      it "should have value which is the model value" do
        @input.should have_value 'model value'
      end

      it "should not override if the value is given" do
        input = Formic::Form.new @page
        input = Formic::Input.new @page, 'some_name', {:value => 'some other value'}
        input.should have_value 'some other value'
      end

      after(:each) do
        @input.to_s
        @form.to_s
      end
    end

    context "form without model" do
      before(:each) do
        @form = Formic::Form.new @page
        @input = Formic::Input.new @page, 'some_name', @options
      end

      it "should have name which is given" do
        @input.should have_name 'some_name'
      end

      it "should be allowed to passed in name" do
        input = Formic::Input.new @page
        input.required :name => 'passed_in_name'
        input.options[:name].should == 'passed_in_name'
      end

      after(:each) do
        @input.to_s
        @form.to_s
      end
    end

    context "#method missing" do
      before(:each) do
        @input = Formic::Input.new @page
        @result = @input.small 'name', @options, &@block
      end

      it "should add the class" do
        @input.should have_class 'small'
      end

      it "should set the field" do
        @input.field.should == 'name'
      end

      it "should return the object" do
        @input.should == @result
      end

      context "with form model" do
        it "should have correct name" do
          form = Formic::Form.new @page
          form.create @model
          mock(@model).send(:some_name) {'model value'}
          input = Formic::Input.new @page
          input.required 'some_name', @options
          input.should have_name 'object[some_name]'
        end
      end

      context "#label" do
        it "should be generated from field" do
          input = Formic::Input.new @page, 'some_name', @options
          input.label.should == 'Some Name'
        end

        it "could be passed in via option" do
          input = Formic::Input.new @page, 'some_name', :label => 'Some Label'
          input.label.should == 'Some Label'
        end
      end
    end
  end
end
