require 'spec_helper'

describe Formic::Tab do
  before(:each) do
    @model = Object.new
    stub(@page = Object.new).url_for(@model) {'/models/1'}
    stub(@page).render

    @tab_panel = Formic::TabPanel.new @page
    @options = {:key => 'value'}
    @block = Proc.new {}
  end

  context "#initialize" do
    before(:each) do
      @tab = Formic::Tab.new @page, 'title', @options, &@block
    end

    it "should have template formic/default/tab" do
      @tab.template.should == 'formic/default/tab'
    end

    it "should have an id" do
      @tab.options[:id].should == 'title_tab'
    end

    it "should have title do" do
      @tab.title.should == 'title'
    end

    context "#tab_panel" do
      it "should find the latest tab panel" do
        @tab.tab_panel.should == @tab_panel
      end

      it "should add itself to the tab_panel tabs" do
        @tab_panel.tabs.last.should == @tab
      end
    end

    after(:each) do
      @tab.to_s
    end
  end

  context "#method_missing" do
    before(:each) do
      @tab = Formic::Tab.new @page
      @tab.style 'title', @options, &@block
    end

    it 'should add the class' do
      @tab.should have_class 'style'
    end

    it 'should have content' do
      @tab.content.should == @block
    end

    it 'should merge options' do
      @tab.options[:key].should == 'value'

    end

    after(:each) do
      @tab.to_s
    end
  end
end
